import matplotlib.pyplot as plt
from .color_print import print_color
from collections import Counter

def detect_warnings_failures(sequences):
    """
    Detect warnings and failures based on the lengths of the sequences.

    :param sequences: list of sequences
    :type sequences: list of str
    :return: True if no warnings or failures detected, False otherwise
    :rtype: bool
    """
    sequence_lengths = [len(seq) for seq in sequences]
    # Detect warning and failure situations
    if 0 in sequence_lengths:
        print_color("X | Some sequences have zero length.","red")
        return False
    if len(set(sequence_lengths)) > 1:
        print_color("- | Not all sequences are of the same length.","yellow")
        return False
    else:
        print_color("O | Lengths of Sequences are good","green")
        return True

# version 2
def run_sld6(sequences,sub_directory_path):
    """
    Execute the sequence analysis for sequence length distribution. This includes plotting the distribution and 
    checking for warnings/failures.

    :param sequences: list of sequences
    :type sequences: list of str
    :param sub_directory_path: The directory path where the plot will be saved
    :type sub_directory_path: str
    :return: True if the analysis passes, False otherwise
    :rtype: bool
    """
    sequence_lengths = [len(seq) for seq in sequences]
    length_freq = Counter(sequence_lengths)
    
    # separate keys and values for plotting, but sort them by sequence length
    if len(length_freq) == 1:
        unique_length = list(length_freq.keys())[0]
        
        # Add the adjacent lengths with frequency 0
        length_freq[unique_length - 1] = 0
        length_freq[unique_length + 1] = 0

    # separate keys and values for plotting, but sort them by sequence length
    lengths, freqs = zip(*sorted(length_freq.items()))
    # lengths = list(length_freq.keys())
    # freqs = list(length_freq.values())
    plt.figure(figsize=(10,6))
    plt.plot(lengths, freqs, marker='o')
    plt.title('Distribution of sequence lengths over all sequences')
    plt.xlabel('Sequence length(bp)')
    plt.ylabel('Count of sequences')
    plt.grid(True)
    plt.savefig(f"{sub_directory_path}/Sequence_Length_Distribution.png")
    # plt.show()

    return detect_warnings_failures(sequences)




import collections
import csv
from .color_print import print_color

def run_os7(dna_sequences,sub_directory_path):
    """
    Perform analysis on overrepresented sequences in the provided DNA sequences. Sequences with representation over
    1% and 0.1% are specifically identified. Also, writes these overrepresented sequences to a CSV file.

    :param dna_sequences: List of DNA sequences to analyze
    :type dna_sequences: list of str
    :param sub_directory_path: Path to the subdirectory where to save the CSV file with overrepresented sequences
    :type sub_directory_path: str
    :return: True if the sequences pass the overrepresentation check, False otherwise
    :rtype: bool
    """
    # Track only the first 100,000 sequences
    dna_sequences = dna_sequences[:100000] if len(dna_sequences)>100000 else dna_sequences

    # Truncate sequences over 75bp to 50bp for analysis
    dna_sequences = [seq[:50] if len(seq) > 75 else seq for seq in dna_sequences]

    total_count = len(dna_sequences)
    
    counter = collections.Counter(dna_sequences)
    
    table = []
    
    sequences_exceeding_1_percent = 0
    sequences_exceeding_0_1_percent = 0

    for sequence, count in counter.items():
        percentage = (count / total_count) * 100

        if percentage > 1:
            sequences_exceeding_1_percent += 1
            table.append([sequence, count, percentage])

        elif percentage > 0.1:
            sequences_exceeding_0_1_percent += 1
            table.append([sequence, count, percentage])

    with open(f"{sub_directory_path}/overrepresented_sequences.csv", 'w', newline='') as f:
        writer = csv.writer(f)
        writer.writerow(["Sequence", "Count", "Percentage"])
        writer.writerows(table)

    if (sequences_exceeding_1_percent > 0):
        print_color(f"X | Overrepresented sequences NOT pass. {sequences_exceeding_1_percent} sequences representing more than 1% of the total", "red")
        return False
    elif (sequences_exceeding_0_1_percent > 0):
        print_color(f"- | Overrepresented sequences warning. {sequences_exceeding_0_1_percent} sequences representing more than 0.1% of the total", "yellow")
        return False
    else:
        print_color(f"O | Overrepresented sequences pass. No sequences representing more than 0.1% of the total", "green")
        return True



import pytest
import os
import numpy as np
from ezqc.pbsq1 import run_pbsq1
from ezqc.psqs2 import run_psqs2
from ezqc.pbsc3 import run_pbsc3
from ezqc.psgc4 import run_psgc4
from ezqc.pbnc5 import run_pbnc5
from ezqc.sld6 import run_sld6
from ezqc.os7 import run_os7
from ezqc.ac8 import run_ac8

def parse_fastq_file(file_obj):
    while True:
        header = file_obj.readline().strip()
        if not header:
            break
        sequence = file_obj.readline().strip()
        file_obj.readline()
        quality_str = file_obj.readline().strip()
        yield header, sequence, quality_str

headers = []
sequences = []
quality_strings = []

directory_name = "ezqc_output"
# Get the current working directory
current_directory = os.getcwd()
# Create a path for the new directory
new_directory_path = os.path.join(current_directory, directory_name)

# Create the directory if it doesn't already exist
if not os.path.exists(new_directory_path):
    os.makedirs(new_directory_path)
    print(f"Directory '{directory_name}' created successfully!")
else:
    print(f"Directory '{directory_name}' already exists.")


# test 1
with open("tests/SRR020192.fastq", 'r') as fastq_file:
     for header, sequence, quality_str in parse_fastq_file(fastq_file):
        headers.append(header)
        sequences.append(sequence)
        quality_strings.append(quality_str)

def test1_function1_FALSE():
    assert run_pbsq1(quality_strings,directory_name) == False
def test1_function2_TRUE():
    assert run_psqs2(quality_strings,directory_name) == True
def test1_function3_FALSE():
    assert run_pbsc3(sequences,directory_name) == False
def test1_function4_FALSE():
    assert run_psgc4(sequences,directory_name) == False
def test1_function5_TRUE():
    assert run_pbnc5(sequences,directory_name) == True
def test1_function6_FALSE():
    assert run_sld6(sequences,directory_name) == False
def test1_function7_FALSE():
    assert run_os7(sequences,directory_name) == False
def test1_function8_TURE():
    assert run_ac8(sequences,directory_name) == True

#test 2
with open("tests/SRR24755455.fastq", 'r') as fastq_file:
     for header, sequence, quality_str in parse_fastq_file(fastq_file):
        headers.append(header)
        sequences.append(sequence)
        quality_strings.append(quality_str)

def test2_function1_FALSE():
    assert run_pbsq1(quality_strings,directory_name) == False
def test2_function2_TRUE():
    assert run_psqs2(quality_strings,directory_name) == True
def test2_function3_FALSE():
    assert run_pbsc3(sequences,directory_name) == False
def test2_function4_FALSE():
    assert run_psgc4(sequences,directory_name) == False
def test2_function5_TRUE():
    assert run_pbnc5(sequences,directory_name) == True
def test2_function6_FALSE():
    assert run_sld6(sequences,directory_name) == False
def test2_function7_FALSE():
    assert run_os7(sequences,directory_name) == False
def test2_function8_TURE():
    assert run_ac8(sequences,directory_name) == True


import argparse
import numpy as np
from .pbsq1 import run_pbsq1
from .psqs2 import run_psqs2
from .pbsc3 import run_pbsc3
from .psgc4 import run_psgc4
from .pbnc5 import run_pbnc5
from .sld6 import run_sld6
from .os7 import run_os7
from .ac8 import run_ac8
import os

def parse_fastq_file(file_obj):
    """
    This function reads a FASTQ file and yields the header, sequence and quality string line by line.

    :param file_obj: A file object of a FASTQ file to be parsed.
    :type file_obj: _io.TextIOWrapper
    :yield: header, sequence and quality string from the FASTQ file.
    :rtype: tuple
    """
    while True:
        header = file_obj.readline().strip()
        if not header:
            break
        sequence = file_obj.readline().strip()
        file_obj.readline()
        quality_str = file_obj.readline().strip()
        yield header, sequence, quality_str

def main():
    """
    This function is the main driver of the program. It accepts FASTQ files as input and generates quality analysis 
    results as output.

    It does so by taking user inputs (FASTQ files and an optional output directory) via command line arguments. It 
    then creates necessary directories for output files. Finally, it processes each input FASTQ file using the 
    parse_fastq_file function, performs quality analysis on the data, and writes the results to the output directory.
    """
    logo = r'''
    ███████╗███████╗ ██████╗  ██████╗
    ██╔════╝╚══███╔╝██╔═══██╗██╔════╝
    █████╗    ███╔╝ ██║   ██║██║     
    ██╔══╝   ███╔╝  ██║▄▄ ██║██║     
    ███████╗███████╗╚██████╔╝╚██████╗
    ╚══════╝╚══════╝ ╚══▀▀═╝  ╚═════╝ made by Tinger Shi & Sky Li
    '''
    
    print(logo)

    # Create the argparse object and define the arguments
    parser = argparse.ArgumentParser(description="EZQC FastQ Quality Analyzer")
    parser.add_argument('seqs', metavar='SEQ', nargs='+', help='input .fastq file(s)')
    parser.add_argument('-o', '--output', default='ezqc_output', help='output directory (default: ezqc_output)')

    # Parse the command-line arguments 
    args = parser.parse_args()

    directory_name = args.output
    current_directory = os.getcwd()
    new_directory_path = os.path.join(current_directory, directory_name)

    # Create the directory if it doesn't already exist
    if not os.path.exists(new_directory_path):
        os.makedirs(new_directory_path)
        print(f"Directory '{directory_name}' created successfully!")
    else:
        print(f"Directory '{directory_name}' already exists.")

    total_num = 0
    pass_num = 0


    # Process each .fastq file
    for file_path in args.seqs:
        print(f"Processing file: {file_path}")
       
        file_base_name = os.path.basename(file_path)
        file_base_name = os.path.splitext(file_base_name)[0]
        sub_directory_path = os.path.join(new_directory_path, file_base_name)

        # Create the directory if it doesn't already exist
        if not os.path.exists(sub_directory_path):
            os.makedirs(sub_directory_path)
            print(f"Directory '{sub_directory_path}' created successfully!")
        else:
            print(f"Directory '{sub_directory_path}' already exists.")

        total_num += 1
        headers = []
        sequences = []
        quality_strings = []

        with open(file_path, 'r') as fastq_file:
            for header, sequence, quality_str in parse_fastq_file(fastq_file):
                headers.append(header)
                sequences.append(sequence)
                quality_strings.append(quality_str)
        # average_length_each_sequence = int(np.average([len(x) for x in quality_strings]))

        results = [
            run_pbsq1(quality_strings,sub_directory_path),
            run_psqs2(quality_strings,sub_directory_path),
            run_pbsc3(sequences,sub_directory_path),
            run_psgc4(sequences,sub_directory_path),
            run_pbnc5(sequences,sub_directory_path),
            run_sld6(sequences,sub_directory_path),
            run_os7(sequences,sub_directory_path),
            run_ac8(sequences,sub_directory_path)
        ]

    if all(results): 
        pass_num += 1
    print(f"All fastq file analyzied, {pass_num}/{total_num} passed all tests.")

if __name__ == '__main__':
    main()



import matplotlib.pyplot as plt
from .color_print import print_color

adapters = {
    'Illumina Universal Adapter': 'AGATCGGAAGAG',
    'Illumina Small RNA 3\' Adapter': 'TGGAATTCTCGG',
    'Illumina Small RNA 5\' Adapter': 'GATCGTCGGACT',
    'SOLiD Small RNA Adapter': 'CTGCTGTACGGCCAAGGCG'
}

def calculate_adapter_content(sequences, adapters):
    """
    Calculate the adapter content in the provided sequences.

    :param sequences: List of sequences to analyze for adapter content
    :type sequences: list of str
    :param adapters: Dictionary of adapters, where keys are adapter names and values are adapter sequences
    :type adapters: dict
    :return: A dictionary of percentages for each adapter and a boolean indicating if all sequences passed the check
    :rtype: dict, bool
    """

    counts = {name: [0]*len(sequences) for name, seq in adapters.items()}

    curr = 0
    for seq in sequences:
        for name, adapter in adapters.items():
            if len(seq) >= len(adapter):
                for i in range(len(seq) - len(adapter) + 1):
                    if seq[i:i+len(adapter)] == adapter:
                        counts[name][curr] += 1
        curr += 1

    percentages = {name: [count/len(sequences)*100 for count in count_list] for name, count_list in counts.items()}

    all_pass = True
    for name, percentage in percentages.items():
        if max(percentage) > 10:
            print_color(f'X | {name} is present in more than 10% of all reads','red')
            all_pass = False
        elif (max(percentage) > 5):
            print_color(f'- | {name} is present in more than 5% of all reads','yellow')
            all_pass = False
    if (all_pass):
        print_color(f'O | No adaptor is present in more than 5% of all reads','green')

    return percentages, all_pass

def plot_adapter_content(adapter_percentages,sub_directory_path):
    """
    Plot the adapter content percentages.

    :param adapter_percentages: A dictionary of percentages for each adapter
    :type adapter_percentages: dict
    :param sub_directory_path: Path to the subdirectory where to save the plot image
    :type sub_directory_path: str
    """
    plt.figure()
    positions = list(range(1, max(map(len, adapter_percentages.values())) + 1))

    for name, percentages in adapter_percentages.items():
        plt.plot(positions[:len(percentages)], percentages, label=name)

    plt.xlabel('Position in read (bp)')
    plt.ylabel('% Adapter')
    plt.ylim(0, 100)
    plt.title('Adapter Content')
    plt.legend()

    plt.savefig(f"{sub_directory_path}/adaptor_content_plot.png")
    #plt.show()

def run_ac8(seqs,sub_directory_path):
    """
    Run the adapter content calculation and plot for the provided sequences.

    :param seqs: List of sequences to analyze for adapter content
    :type seqs: list of str
    :param sub_directory_path: Path to the subdirectory where to save the plot image
    :type sub_directory_path: str
    :return: True if all sequences passed the adapter content check, False otherwise
    :rtype: bool
    """
    adapter_percentages, all_pass = calculate_adapter_content(seqs, adapters)
    plot_adapter_content(adapter_percentages,sub_directory_path)
    return all_pass